-- Greg Stitt
-- University of Florida
--
-- Description: This entity adds 4 inputs represented by an
-- array. NOTE: This isn't a good way of implementing this functionality and is
-- solely used to demonstrate arrays and packages. A series of four adders is
-- not efficient, and should be replaced with a adder tree (possibly with
-- pipelining). Also, this implementation ignores overflow.
--
-- Topics discussed: Arrays used for I/O, packages

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.sum_pkg.all;                   -- include our custom package

-- In this entity, we want to add 4 inputs to produce the output. Instead of
-- manaully defining 4 inputs, ideally we could use an array. In previous
-- examples, we defined an array by creating a new type. We can potentially do
-- that here, but where would we define the type for the array? We previously
-- defined the type in the architecture, but here we need the definition for
-- the entity. To accomplish this, we use a package (sum_pkg), which is
-- analagous to a header file in C/C++. If you look in sum_pkg.vhd, you will
-- see a type definition for input_array, which defines it as an array of 4,
-- 8-bit std_logic_vectors. Notice that custom packages are part of the "work"
-- directory.

entity sum is
  port (
    inputs     : input_array;
    output     : out std_logic_vector(7 downto 0));
end sum;

architecture BHV of sum is
begin  -- BHV

  process(inputs)
    variable temp_sum : unsigned(7 downto 0);    
  begin

    temp_sum := to_unsigned(0, temp_sum'length);

    -- with the input array, we can use a for loop to iterate across all inputs
    -- IMPORTANT: This is not an efficient way of adding 4 numbers. This
    -- example is intentionally simple to illustrate arrays and packages.

    for i in 0 to 3 loop
      temp_sum := temp_sum + unsigned(inputs(i));      
    end loop;

    output <= std_logic_vector(temp_sum);
    
  end process;
end BHV;
